import requests,hmac, hashlib, base64, json, re
from datetime import datetime

class UnleashedBase:
	"""
		Base class for making a connection to API of unleashedsoftware.com
		Inputs:
			auth_id : Your user account Authorization ID assigned by Unleashed
			auth_sig: Your user account Authorization signature assigned by Unleashed
			api_add: the address of the api access to Unleashed, typically https://api.unleashedsoftware.com
	"""

	def __init__(self, auth_id, auth_sig,  api_add):
		self.header = {'Content-Type': 'application/json', \
			'Accept':'application/json',\
			'api-auth-id':auth_id,
			'api-auth-signature':None}
		self.auth_sig = auth_sig
		self.auth_id = auth_id
		self.api_add = api_add

	@staticmethod
	def getSignature(args, privateKey):
		# print(args)
		key = str.encode(privateKey, encoding='ASCII')
		msg = str.encode(args, encoding='ASCII')
		myhmacsha256 = hmac.new(key, msg, digestmod=hashlib.sha256).digest()
		return base64.b64encode(myhmacsha256).decode()

class Resource(UnleashedBase):

	def __init__(self, resource_name, auth_id, auth_sig,  api_add, **kwargs):
		super().__init__(auth_id, auth_sig,  api_add)
		# Create the filter:
		self.resource_name = resource_name
		self.filter = ''
		for name, value in kwargs.items():
			if self.filter == '':
			# print('{0}={1}'.format(name, value))
				self.filter += '{0}={1}'.format(name, value)
			else:
				self.filter += '&{0}={1}'.format(name, value)
		# print(self.filter)
		if self.filter is not None:
			self.header['api-auth-signature'] = self.getSignature(self.filter,self.auth_sig)
			self.address = self.api_add+'/'+self.resource_name+'?'+self.filter
		else:
			self.header['api-auth-signature'] = self.getSignature('',self.auth_sig)
			self.header = self.header
			self.address = self.api_add+'/'+self.resource_name

	def first_page(self):
		"""
			Given a product object, this method will return the first page
			of paginated results that Unleashed gives back with the git request

			Returns:
				json object containing results from first page of git request
		"""
		# print(self.address, self.header)
		return(json.dumps(requests.get(self.address, headers = self.header).json()['Items']))

	def all_results(self):
		pages = self.getPages()
		results = []
		for i in range(1,pages + 1):
			r = requests.get(self.address, headers = self.header).json()['Items']
			for result in r:
				results.append(result)
		return(json.dumps(results))

	def getPages(self):

		return(requests.get(self.address, headers = self.header).json()['Pagination']['NumberOfPages'])

class EditableResource(Resource):

	def __init__(self, resource_name, auth_id, auth_sig,  api_add, **kwargs):
		super().__init__(resource_name, auth_id, auth_sig,  api_add, **kwargs)

	def post_object(self, guid, object):
		"""
			submit object to unleashed.
		"""
		return(requests.post(self.address, headers = self.header, data = object))



"""unleashed-py - Python package to connect to the Unleashed Software inventory management API"""
import sys, os
__version__ = '0.0.13'
__author__ = 'Jonathan Mucha <jonmucha@gmail.com>'
sys.path.append(os.path.join(os.path.dirname(__file__)))

# __all__ = []
from .unleashed_py import UnleashedBase
from .unleashed_py import Resource
from .unleashed_py import EditableResource



from __future__ import absolute_import
from __future__ import print_function

import io
import os
import re
from glob import glob
from setuptools import find_packages
from setuptools import setup
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

def read(filename):
    filename = os.path.join(os.path.dirname(__file__), filename)
    text_type = type(u"")
    with io.open(filename, mode="r", encoding='utf-8') as fd:
        return re.sub(text_type(r':[a-z]+:`~?(.*?)`'), text_type(r'``\1``'), fd.read())


setup(
    name="unleashed-py",
    version="0.0.13",
    url="https://github.com/jonathanStrange0/unleashed-py",
    license='MIT',

    author="Jonathan Mucha",
    author_email="jonmucha@gmail.com",

    long_description_content_type="text/x-rst",
    description="Python package to connect to the Unleashed Software inventory management API",
    long_description=read("README.rst"),
    packages=find_packages('unleashed-py'),
    package_dir={'': 'unleashed-py'},
    py_modules=[splitext(basename(path))[0] for path in glob('unleashed-py/*.py')],
    include_package_data=True,
    # packages=find_packages(exclude=('tests',)),

    install_requires=['requests'],

    python_requires='>=3.6',

    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
